# -*- coding: utf-8 -*-
'''
Created on 05.12.2010

@author: Sergey Prokhorov <root@seriyps.ru>
'''
import json
from gwibber.microblog.util.const import *
from gwibber.microblog.util import log
from gwibber.microblog import util
import time
import urllib
from vk_api_wrapper import vk_api, vkException
# Try to import * from custom, install custom.py to include packaging 
# customizations like distro API keys, etc
try:
  from gwibber.microblog.util.custom import *
except:
  pass

log.logger.name="Vkontakte"

VK_APP_ID="2036925"#XXX: remove me! see gwibber.microblog.util.const.VK_APP_ID

PROTOCOL_INFO={
  "name": "Vkontakte",
  "version": "0.1",

  "config": [
    "color",
    "receive_enabled",
    "receive_groups",
    "send_enabled",
    "username",
    "uid",
    "private:access_token"
  ],

  "authtype": "vkontakte",
  "color": "#45688E", # #6D8FB3

  "features": [
    "send",
    "reply",
    "receive",
    #"thread",
    "delete",
    #"send_thread",
    "like",
    "sincetime"
  ],

  "default_streams": [
    "receive",
  ]
}

URL_PREFIX='http://vkontakte.ru/'

class attachment_processor:
  """Generate attachment fields
  http://vkontakte.ru/developers.php?o=-1&p=%D0%9E%D0%BF%D0%B8%D1%81%D0%B0%D0%BD%D0%B8%D0%B5+%D0%BF%D0%BE%D0%BB%D1%8F+attachment
  """

  def __init__(self, client):
    self.client=client

  def _process_photo(self, attachment):
    return "photo", {"url":"%sphoto%d_%d"%(URL_PREFIX, attachment["owner_id"], attachment["pid"]),
            "picture": attachment["src"]}

  _process_posted_photo=_process_photo

  def _process_graffiti(self, attachment):
    return "photo", {"url":"%sgraffiti%d?from_id=%d"%(URL_PREFIX, attachment["gid"], attachment["owner_id"]),
            "picture": attachment["src"]}

  def _process_video(self, attachment):
    lnk="%svideo%d_%d"%(URL_PREFIX, attachment["owner_id"], attachment["vid"])
    video_data=self.client.api.video.get(videos="%d_%d"%(attachment["owner_id"], attachment["vid"]))["response"]
    return "video", {"url": lnk,
                     "name": attachment["title"],
                     "source": lnk,
                     "picture": video_data[1]["image"]}

  def _process_link(self, attachment):
    l={"name": attachment["title"],
       "url": attachment["url"],
       "description": attachment["description"]
       }
    if attachment.has_key("image_src"):
      l["picture"]=attachment["image_src"]
    return "link", l

  def process(self, attachment, message):
    """find is processor has matched handler, and if yes,
    generate attachment with this handler and merge it with
    original message"""
    type=attachment["type"]
    processor_name="_process_%s"%type
    if hasattr(self, processor_name):
      key, val=getattr(self, processor_name)(attachment[type])
      message.update({key: val})
    return message


###########
# MAGICK!!!
###########
'''vkontakte API don't return comments for wall posts,
but return comments count, so, we need download comments
for each posts with comments count > 0 by separate API query.
But, surprise! API don't return commenters profile data, only
commenter ID. That means, that we need download commenter profile
for each commenter. Only one good thing is that we can
download many profiles by single query. So, when process
wall posts, we insert "lasy_mask" fake objects in places, where
commenter profile must be. And ALL profiles loads only when
some slave object need profile data.
'''
class lazy_master:
  '''Dispatcher, that store slaves and start
  worker when one of slaves need data'''

  def __init__(self, worker):
    '''
    @param worker: callback function, that do main work (convert
    masks to results)'''
    self.masks={}
    self.results={}
    self.worker=worker

  def new_mask(self, id):
    mask=lazy_mask(self, id)
    self.masks[mask.id]=mask
    return mask

  def run(self):
    #TODO: implement multiple run
    self.results=self.worker(self.masks)

  def get(self, id):
    if not self.results:#.get(id, False):
      self.run()
    return self.results.get(id, None)


class lazy_mask(object):
  '''Fake object, which is substituted in
  place of masked object'''

  def __init__(self, master, id):
    self.master=master
    self.id=id
    self._result=None

  def _res(self):
    '''Run lazy job'''
    if not self._result:
      self._result=self.master.get(self.id)
    return self._result

  def __getattribute__(self, name):
    '''proxy all queries to masked object'''
    name=name.replace('_lazy_mask', '')
    #print 'attr', name
    if name in ['_result', '_res', 'master', 'id']:#don't proxy requests for own properties
      return super(lazy_mask, self).__getattribute__(name)
    else:#but proxy requests for masked object
      return self._res().__getattribute__(name)

  def __getitem__(self, key):
    '''provide object["key"] access. Else can raise 
    TypeError: 'lazy_mask' object is unsubscriptable'''
    return self._res().__getitem__(key)
###########
# END MAGICK!!!
###########


class Client:

  def __init__(self, acct):
    self.account=acct
    self.session=json.loads(acct.get("access_token"))
    self.api=vk_api(session=self.session["sid"],
                    secret=self.session["secret"],
                    mid=acct.get("uid"),
                    app_id=VK_APP_ID)
    self.attachment_processor=attachment_processor(self)

  def __call__(self, opname, **args):
    try:
      return getattr(self, opname)(**args)
    except vkException, e:
      return [self._format_error(e)]


  def _format_error(self, e, type=None, message=None):
    log.logger.error('Vkontakte error #%d: "%s" in url %s', e.code, e.msg, e.url)
    if e.code in (3, 5, 7):
      _type="auth"
    #elif e.code in (2, 4, 100):
    #  _type="internal"
    else:
      _type="unknown"
    return {"error": {"type": type or _type,
                       "account": self.account,
                       "message": message or e.msg}}

  def _user(self, acc_data):
    """Extract single user data from API response"""
    if acc_data.has_key("gid"):
      return {
              "name": acc_data["name"],
              "id": str(acc_data["gid"]*-1),
              "is_me": False,
              "image": acc_data["photo"],
              "url": "%sclub%s"%(URL_PREFIX, acc_data["gid"])
              }
    else:
      return {
              "name": "%s %s"%(acc_data["first_name"], acc_data["last_name"]),
              "id": str(acc_data["uid"]),
              "is_me": acc_data["uid"]==int(self.account["uid"]),
              "image": acc_data["photo"],
              "url": "%sid%s"%(URL_PREFIX, acc_data["uid"])
              }

  def _message(self, item, account, comment_author_factory):
    """Extract single message data from API response"""
    m={}
    m["mid"]=str(item["post_id"])
    m["service"]="vkontakte"
    m["account"]=self.account["id"]
    m["time"]=item["date"]+time.timezone#XXX: WTF?!? why we don't move timezone correction to presentation logic??? We must store to DB absolute UTC timestamp!!!
    m["sender"]=self._user(account)
    m["url"]="%swall%s_%s"%(URL_PREFIX, m["sender"]["id"], item["post_id"])
    m["text"]=item["text"]
    m["html"]=util.linkify(item["text"], escape=False)
    m["content"]=m["html"]

    if item.get("likes", 0):
      m["likes"]={}
      m["likes"]["count"]=item["likes"]["count"]

    if item.has_key("attachment"):
      if item["attachment"]["type"]=="audio":
        return#FIXME: Gwibber don't support audio
      m=self.attachment_processor.process(item["attachment"], m)

    if item["comments"]["count"]>0:
      #log.logger.debug("Post %s has %d"%(m["url"], item["comments"]["count"]))
      try:
        comments=self.api.wall.getComments(owner_id=m["sender"]["id"],
                                           post_id=item["post_id"],
                                           sort="asc",
                                           count=50)
      except vkException, e:
        return self._format_error(e)
      if comments.has_key("response"):
        comm_list=[]
        for comment in comments["response"]:
          if not isinstance(comment, dict):#drop first(?) element, because it is integer count of comments
            continue
          comm_list.append({"text":comment["text"],
                    "time": comment["date"],
                    "sender": comment_author_factory.new_mask(comment["uid"])#lazy magick. See notes upper.
                    })
        m["comments"]=comm_list
        #log.logger.debug("For post %s loaded %d comments"%(m["url"], len(comm_list)))
    return m

  def _get_lazy_profiles_factory(self):
    def lazy_comment_profiles_loader(slaves):
      '''magick (see comments upper) lazy worker (transform map of
      profile id's to map of profiles'''
      ids=[]#slaves.keys()
      for slave in slaves.itervalues():
        ids.append(str(slave.id))
      uids=",".join(ids)
      try:
        profiles=self.api.getProfiles(uids=uids, fields="uid,photo,first_name,last_name")
      except vkException, e:
        self._format_error(e)#log message to console
        return {}
      res={}
      for profile in profiles['response']:
        res[profile["uid"]]=self._user(profile)
      return res

    return lazy_master(worker=lazy_comment_profiles_loader)

  def receive(self, since=None):
    '''Retrieve messages from friend's and own walls.'''
    if not since:
      since=int(time.time()-60*60*24)#24 hours
    response=self.api.newsfeed.get(filters="post", start_time=since)["response"]

    comment_author_factory=self._get_lazy_profiles_factory()

    profiles_by_id={}#index user and group profiles by user id
    for _acc in response["profiles"]:
      profiles_by_id[_acc["uid"]]=_acc
    if self.account["receive_groups"]:
      for _acc in response["groups"]:
        profiles_by_id[_acc["gid"]*-1]=_acc
    msgs=[]
    for item in response["items"]:
      if item["source_id"]<0 and not self.account["receive_groups"]:#source_id > 0 for peoples and < 0 for groups
        continue#skip groups if "receive_groups" option disabled
      account=profiles_by_id[item["source_id"]]
      message=self._message(item, account, comment_author_factory)
      msgs.append(message)
      #yield message #api don't return post comments and broke yield (((
    for msg in self._own_wall_posts(30, comment_author_factory):
      msgs.append(msg)
    comment_author_factory.run()#run lazy job
    return msgs

  def _own_wall_posts(self, count, comment_author_factory):
    '''Download all posts user post in own wall'''
    try:
      response=self.api.wall.get(filter="owner", count=count)["response"]
    except vkException, e:
      yield self._format_error(e)
      return
    me=None
    for post in response:
      if not isinstance(post, dict):#skip first element - number of items
        continue
      post["post_id"]=post["id"]
      post["source_id"]=post["from_id"]
      if not me:
        me=self.api.getProfiles(uids=post["source_id"], fields="uid,photo,first_name,last_name")["response"][0]
      yield self._message(post, me, comment_author_factory)

  def user_messages(self, id=None, count=util.COUNT, since=None):
    """Messages posted by some user"""
    raise NotImplementedError
    if count>100:#vk limit
      count=100
    comment_author_factory=self._get_lazy_profiles_factory()
    return self._own_wall_posts(count, comment_author_factory)

  def send(self, message):
    self.api.wall.post(message=message)
    return self._own_wall_posts(1, self._get_lazy_profiles_factory())

  def send_thread(self, message, target):
    """Send comment with text @message to post @target
    @param message: message text
    @param target: target post"""
    self.api.wall.addComment(owner_id=target["sender"]["id"],
                               post_id=target["mid"],
                               text=message)
    return []

  def like(self, message):
    """Like message
    @param message: target post you like"""
    self.api.wall.addLike(owner_id=message["sender"]["id"],
                            post_id=message["mid"])
    return []

  def delete(self, message):
    """Remove message from you wall
    @param message: post you want to delete"""
    self.api.wall.delete(owner_id=message["sender"]["id"],
                           post_id=message["mid"])
    return []
