
import gtk
from gwibber.microblog.util import resources
from gwibber.microblog.util.const import *
try:
  from gwibber.microblog.plugins.vkontakte.vk_api_wrapper import vk_api
except ImportError:
  try:
    from vkontakte.vk_api_wrapper import vk_api
  except ImportError:
    vk_api=None

import gnomekeyring
import webkit
import urllib
import json

import gettext
from gettext import gettext as _
if hasattr(gettext, 'bind_textdomain_codeset'):
    gettext.bind_textdomain_codeset('gwibber', 'UTF-8')
gettext.textdomain('gwibber')

VK_APP_ID="2036925"#XXX: remove me! see gwibber.microblog.util.const.VK_APP_ID

class AccountWidget(gtk.VBox):
  """AccountWidget: A widget that provides a user interface for configuring
  Vkontakte accounts in Gwibber"""

  def __init__(self, account=None, dialog=None):
    """Creates the account pane for configuring Vkontakte accounts"""
    gtk.VBox.__init__(self, False, 20)
    self._state_init()
    if account:
      self.account=account
    else:
      self.account={}
    self.dialog=dialog
    has_access_token=True
    if self.account.has_key("id"):#check is current account already authorized
      try:
        value=gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, {"id": str("%s/%s"%(self.account["id"], "access_token"))})[0].secret
      except gnomekeyring.NoMatchError:
        has_access_token=False
    try:#if account authorized, don't show "authorize" button
      if self.account["access_token"] and self.account["username"] and has_access_token and not self.dialog.condition:
        self._state_authorized()
      else:#else don't show "authorized" label
        self._state_not_authorized()
    except:
      self._state_not_authorized()


  def on_vk_auth_clicked(self, widget, data=None):
    """Start embed browser and show authorization dialog"""
    web=webkit.WebView()
    web.get_settings().set_property("enable-plugins", False)
    web.load_html_string(_("<p>Please wait...</p>"), "file:///")

    url=urllib.urlencode({#call to vkontakte API authorization 
      "app": VK_APP_ID,
      "layout": "popup",
      "type": "browser",
      "settings": 1024+8192+16,
    })
    web.set_size_request(450, 340)
    web.load_uri("http://vkontakte.ru/login.php?"+url)
    web.connect("title-changed", self.on_vk_auth_title_change)
    self._state_show_browser(web)

  def on_vk_auth_title_change(self, web=None, title=None, data=None):
    """When user confirm or revoke authorization in embed browser"""
    saved=False
    url=web.get_main_frame().get_uri()
    if title.get_title()=="Login success" and "session=" in url:
      """When user successfully authorize our application, extract session secrets..."""
      json_string=urllib.unquote(url.split("session=")[1])
      """ "http://vkontakte.ru/api/login_success.html#session=%7Bsome_json%7D" > "%7Bsome_json%7D" > {some_json} """
      self.account["access_token"]=json_string#!!! store vk API "session" json string as secret keyring
      data=json.loads(json_string)
      if isinstance(data, dict):
        if data.has_key("mid") and data.has_key("sid") and data.has_key("secret"):
          self.account["uid"]=data["mid"]
          if vk_api:#try retrieve user name and last name from API
            profile=vk_api(session=data["sid"],
                       secret=data["secret"],
                       mid=data["mid"],
                       app_id=VK_APP_ID).getProfiles(uids=data["mid"], fields="first_name,last_name")
            self.account["username"]=u"%(first_name)s %(last_name)s"%(profile['response'][0])
          else:
            self.account["username"]="id%s"%data["mid"]
          saved=self.dialog.on_edit_account_save()#store user account data
      self._state_authorized()
      self._state_login_success(saved)
      self._state_hide_browser(web)
    if title.get_title()=="Login failure":
      self._state_not_authorized()
      self._state_login_failed()
      self._state_hide_browser(web)

  ### UI-manipulation methods ###

  def _state_init(self):
    """create UI from UI file"""
    self.ui=gtk.Builder()
    self.ui.set_translation_domain("gwibber")
    self.ui.add_from_file(resources.get_ui_asset("gwibber-accounts-vkontakte.ui"))
    self.ui.connect_signals(self)#attach events (on_vk_auth_clicked)
    self.vbox_settings=self.ui.get_object("vbox_settings")
    self.pack_start(self.vbox_settings, False, False)
    self.show_all()

  def _state_show_browser(self, web):
    """Show webkit widget, hide settings etc.."""
    (self.win_w, self.win_h)=self.window.get_size()
    scroll=gtk.ScrolledWindow()
    scroll.add(web)
    self.pack_start(scroll, True, True, 0)
    self.show_all()
    self.ui.get_object("vbox1").hide()
    self.ui.get_object("vbox_advanced").hide()

  def _state_hide_browser(self, web):
    """Hide webkit, show advanced settings, etc.."""
    web.hide()
    self.window.resize(self.win_w, self.win_h)
    self.ui.get_object("vbox1").show()
    self.ui.get_object("vbox_advanced").show()

  def _state_login_success(self, saved):
    """Show 'update' or 'create' buttons if not saved automatically"""
    if self.dialog.ui and self.account.has_key("id") and not saved:
      self.dialog.ui.get_object("vbox_save").show()
    elif self.dialog.ui and not saved:
      self.dialog.ui.get_object("vbox_create").show()

  def _state_login_failed(self):
    """When user fail or revoke authorization, show popup message"""
    gtk.gdk.threads_enter()
    d=gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
      gtk.BUTTONS_OK, _("Vkontakte authorization failed. Please try again."))
    if d.run(): d.destroy()
    gtk.gdk.threads_leave()

  def _state_not_authorized(self):
    """Show 'Authorize with vkontakte' button"""
    self.ui.get_object("hbox_vk_auth_done").hide()
    self.ui.get_object("hbox_vk_auth").show()
    if self.dialog.ui:
      self.dialog.ui.get_object('vbox_create').hide()

  def _state_authorized(self):
    """Hide 'Authorize with vkontakte' button, show 'Authorized'"""
    self.ui.get_object("hbox_vk_auth").hide()
    self.ui.get_object("vk_auth_done_label").set_label(_("%s has been authorized by Vkontakte")%str(self.account["username"]))
    self.ui.get_object("hbox_vk_auth_done").show()
