# -*- coding: utf-8 -*-
'''
Created on 07.12.2010

@author: Sergey Prokhorov <root@seriyps.ru>
'''
from gwibber.microblog import network
from gwibber.microblog.util import log
import hashlib
import urllib
import time

class vkException(Exception):

  def __init__(self, code, msg, url, response):
    self.code=code
    self.msg=msg
    self.response=response
    self.url=url
    Exception.__init__(self, code, msg, url, response)


class vk_api:
  """Vkontakte.ru (vk.com) API wrapper for Gwibber
  == Usage ==
  Initialization:
    api=vk_api(session, secret, mid, app_id)
    "session", "secret", "mid" can be retrieved from call to http://vkontakte.ru/login.php
    "app_id" is an application id from application administrative panel
    (see http://vk.com/developers.php?o=-17680044&p=Desktop+Application+Authorization)
  Make calls to simple methods (has no dots in name, like "getProfiles", "isAppUser"):
    res=api.method(arg_name1=arg1, arg_name2=arg2)
    eg res=api.getProfiles(uids="535397,1", fields="uid,first_name,last_name")
  Make calls to namespaced methods (has dots in name, like "wall.post", "newsfeed.get"):
    res=api.namespace.method(arg_name1=arg1, arg_name2=arg2)
    eg res=api.wall.post(message="New wall post from API")
  Alternative call syntax (better performance):
    res=api._load('namespace.method', arg_name1=arg1, arg_name2=arg2)
    eg res=api._get('wall.post', message="New wall post from API")
  """

  api_url='http://api.vkontakte.ru/api.php'
  version='3.0'

  def __init__(self, session, secret, mid, app_id):
    self._session=session
    self._secret=secret
    self._mid=mid
    self._app_id=app_id
    self._prefix=[]

  def _make_sig(self, params):
    """Create request signature"""
    params="".join("%s=%s"%(k, v) for k, v in sorted(params.items()))
    sig_str=unicode(self._mid)+unicode(params)+unicode(self._secret)
    #log.logger.debug("<vkontakte:sig> %s"%sig_str)
    return hashlib.md5(sig_str).hexdigest()

  def _load(self, method, **kwargs):
    """Make call by method name and arguments"""
    params=dict(kwargs, **{'api_id': str(self._app_id),
                           'method': method,
                           'v': self.version,
                           'format': 'JSON'})
    params["sig"]=self._make_sig(params)
    params["sid"]=self._session
    for _i in xrange(3):#this cycle used for API error #6 "Too many requests"
      res=network.Download(self.api_url, params, False).get_json()
      if not res.has_key("error"):
        #log.logger.debug("%s"%res)
        return res

      if res["error"]["error_code"]!=6:
        break
      '''if we get error #6 "Too many requests per second" (vk allow
      max 3rps http://vkontakte.ru/developers.php?o=-1&p=%D0%92%D0%B7%D0%B0%D0%B8%D0%BC%D0%BE%D0%B4%D0%B5%D0%B9%D1%81%D1%82%D0%B2%D0%B8%D0%B5+%D0%BF%D1%80%D0%B8%D0%BB%D0%BE%D0%B6%D0%B5%D0%BD%D0%B8%D1%8F+%D1%81+API )
      try to sleep with small delay and retry request not more then 3x times'''
      time.sleep(0.5)#TODO: play with sleep value
    raise vkException(res["error"]["error_code"],
                          res["error"]["error_msg"],
                          "%s?%s"%(self.api_url, urllib.urlencode(params)),
                          res)


  """Support for api.namespace.method(**kwargs) calls below"""
  def __getattr__(self, name):
    self._prefix.append(name)
    return self

  def __call__(self, **kwargs):
    if self._prefix:
      method=".".join(self._prefix)
      self._prefix=[]
    else:
      method=kwargs.pop("method")
    return self._load(method, **kwargs)
